@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -855,20 +855,35 @@ const char *CmdExec::FormatPrompt(const char *scan)
       cwd="~";
    {
       const char *home=session->GetHome();
-      if(home && strcmp(home,"/") && !strncmp(cwd,home,strlen(home))
-      && (cwd[strlen(home)]=='/' || cwd[strlen(home)]==0))
+      int home_len=xstrlen(home);
+      if(home_len>1 && !strncmp(cwd,home,home_len)
+      && (cwd[home_len]=='/' || cwd[home_len]==0))
       {
-	 cwd=xstring::format("~%s",cwd+strlen(home));
+	 cwd=xstring::format("~%s",cwd+home_len);
       }
    }
    const char *cwdb=session->GetCwd();
-   cwdb=session->GetCwd();
    if(cwdb==0 || cwdb[0]==0)
       cwdb="~";
    const char *p=strrchr(cwdb,'/');
    if(p && p>cwdb)
       cwdb=p+1;
 
+   const char *lcwd=this->cwd->GetName();
+   {
+      const char *home=get_home();
+      int home_len=xstrlen(home);
+      if(home_len>1 && !strncmp(lcwd,home,home_len)
+      && (lcwd[home_len]=='/' || lcwd[home_len]==0))
+      {
+	 lcwd=xstring::format("~%s",lcwd+home_len);
+      }
+   }
+   const char *lcwdb=this->cwd->GetName();
+   p=strrchr(lcwdb,'/');
+   if(p && p>lcwdb)
+      lcwdb=p+1;
+
    static const char StartIgn[]={RL_PROMPT_START_IGNORE,0};
    static const char EndIgn[]={RL_PROMPT_END_IGNORE,0};
 
@@ -887,6 +902,8 @@ const char *CmdExec::FormatPrompt(const char *scan)
       { 'S', slot?slot.get():"" },
       { 'w', cwd },
       { 'W', cwdb },
+      { 'l', lcwd },
+      { 'L', lcwdb },
       { '[', StartIgn },
       { ']', EndIgn },
       { 0, "" }
@@ -1210,7 +1227,7 @@ void CmdExec::RegisterCommand(const char *name,cmd_creator_t creator,const char
       c->creator=creator;
       if(short_desc)
 	 c->short_desc=short_desc;
-      if(long_desc)
+      if(long_desc || strlen(c->long_desc)<2)
 	 c->long_desc=long_desc;
       return;
    }
@@ -1237,23 +1254,33 @@ const char *CmdExec::CmdByIndex(int i)
    return static_cmd_table[i].name;
 }
 
-Job *CmdExec::default_cmd()
+bool CmdExec::load_cmd_module(const char *op)
 {
-   const char *op=args->a0();
-#ifdef WITH_MODULES
    const char *modname=xstring::cat("cmd-",op,NULL);
+   if(module_init_preloaded(modname))
+      return true;
+#ifdef WITH_MODULES
    if(module_load(modname,0,0)==0)
    {
       eprintf("%s\n",module_error_message());
-      return 0;
+      return false;
    }
-   builtin=BUILTIN_EXEC_RESTART;
-   return this;
+   return true;
 #else
    eprintf(_("%s: command `%s' is not compiled in.\n"),op,op);
-   return 0;
+   return false;
 #endif
 }
+
+Job *CmdExec::default_cmd()
+{
+   const char *op=args->a0();
+   if(load_cmd_module(op)) {
+      builtin=BUILTIN_EXEC_RESTART;
+      return this;
+   }
+   return 0;
+}
 Job *CmdExec::builtin_local()
 {
    if(args->count()<2) {
