@@ -26,7 +26,7 @@
 #include "ArgV.h"
 #include "Filter.h"
 #include "alias.h"
-#include "history.h"
+#include "History.h"
 #include "bookmark.h"
 #include "FileGlob.h"
 
@@ -259,6 +259,8 @@ public:
 
    static const char *CmdByIndex(int i);
 
+   void enable_debug(const char *opt=0);
+
    int	 last_bg;
    bool	 wait_all;
 
