@@ -66,6 +66,7 @@ void FileAccess::Init()
    error_code=OK;
    saved_errno=0;
    mkdir_p=false;
+   rename_f=false;
    ascii=false;
    norest_manual=false;
 
@@ -114,6 +115,7 @@ void  FileAccess::Open(const char *fn,int mode,off_t offs)
    pos=offs;
    this->mode=mode;
    mkdir_p=false;
+   rename_f=false;
    Timeout(0);
 
    switch((open_mode)mode)
@@ -147,7 +149,7 @@ const char *FileAccess::StrError(int err)
       return("Error 0");
    case(SEE_ERRNO):
       if(error)
-   	 return str.vset(error.get(),": ",strerror(saved_errno),NULL);
+	 return str.vset(error.get(),": ",strerror(saved_errno),NULL);
       return(strerror(saved_errno));
    case(LOOKUP_ERROR):
       return(error);
@@ -155,29 +157,29 @@ const char *FileAccess::StrError(int err)
       return("Class is not Open()ed");
    case(NO_FILE):
       if(error)
-   	 return str.vset(_("Access failed: "),error.get(),NULL);
+	 return str.vset(_("Access failed: "),error.get(),NULL);
       return(_("File cannot be accessed"));
    case(NO_HOST):
       return(_("Not connected"));
    case(FATAL):
       if(error)
-   	 return str.vset(_("Fatal error"),": ",error.get(),NULL);
+	 return str.vset(_("Fatal error"),": ",error.get(),NULL);
       return(_("Fatal error"));
    case(STORE_FAILED):
       return(_("Store failed - you have to reput"));
    case(LOGIN_FAILED):
       if(error)
-   	 return str.vset(_("Login failed"),": ",error.get(),NULL);
+	 return str.vset(_("Login failed"),": ",error.get(),NULL);
       return(_("Login failed"));
    case(NOT_SUPP):
       if(error)
-   	 return str.vset(_("Operation not supported"),": ",error.get(),NULL);
+	 return str.vset(_("Operation not supported"),": ",error.get(),NULL);
       return(_("Operation not supported"));
    case(FILE_MOVED):
       if(error)
-   	 return str.vset(_("File moved"),": ",error.get(),NULL);
+	 return str.vset(_("File moved"),": ",error.get(),NULL);
       else
-   	 return str.vset(_("File moved to `"),location?location.get():"?","'",NULL);
+	 return str.vset(_("File moved to `"),location?location.get():"?","'",NULL);
    }
    return("");
 }
@@ -214,6 +216,12 @@ void FileAccess::Open2(const char *f,const char *f1,open_mode o)
    cache->FileChanged(this,file1);
 }
 
+void FileAccess::Rename(const char *rfile,const char *to,bool clobber)
+{
+   Open2(rfile,to,RENAME);
+   rename_f=clobber;
+}
+
 void FileAccess::Mkdir(const char *fn,bool allp)
 {
    Open(fn,MAKE_DIR);
@@ -684,13 +692,26 @@ FileAccess *SessionPool::Walk(int *n,const char *proto)
 
 void SessionPool::ClearAll()
 {
-   for(int n=0; n<pool_size; n++)
-   {
-      if(pool[n])
-      {
-	 SMTask::Delete(pool[n]);
-	 pool[n]=0;
+   int pass=0;
+   for(;;) {
+      int left=0;
+      for(int n=0; n<pool_size; n++) {
+	 if(!pool[n])
+	    continue;
+	 if(pass==0)
+	    pool[n]->Disconnect();
+	 if(!pool[n]->IsConnected()) {
+	    SMTask::Delete(pool[n]);
+	    pool[n]=0;
+	 } else {
+	    left++;
+	 }
       }
+      if(left==0)
+	 break;
+      SMTask::Schedule();
+      SMTask::Block();
+      pass++;
    }
 }
 
@@ -856,15 +877,9 @@ DirList::~DirList()
 
 // ListInfo implementation
 ListInfo::ListInfo(FileAccess *s,const char *p)
-   : FileAccessOperation(s)
+   : FileAccessOperation(s), exclude_prefix(0), exclude(0), need(0),
+   follow_symlinks(false), try_recursive(false), is_recursive(false)
 {
-   exclude=0;
-   exclude_prefix=0;
-
-   need=0;
-
-   follow_symlinks=false;
-
    if(session && p)
    {
       saved_cwd=session->GetCwd();
