@@ -134,6 +134,7 @@ protected:
    int max_retries;
 
    bool	 mkdir_p;
+   bool	 rename_f;
 
    int	 saved_errno;
 
@@ -227,7 +228,7 @@ public:
    void AsciiTransfer() { ascii=true; }
    virtual void Close();
 
-   void Rename(const char *rfile,const char *to) { Open2(rfile,to,RENAME); }
+   void Rename(const char *rfile,const char *to,bool clobber=false);
    void Link(const char *f1,const char *f2) { Open2(f1,f2,LINK); }
    void Symlink(const char *f1,const char *f2) { Open2(f1,f2,SYMLINK); }
    void Mkdir(const char *rfile,bool allpath=false);
@@ -473,6 +474,8 @@ protected:
 
    unsigned need;
    bool follow_symlinks;
+   bool try_recursive;
+   bool is_recursive;
 
    void PrepareToDie();
    ~ListInfo();
@@ -481,9 +484,11 @@ public:
    ListInfo(FileAccess *session,const char *path);
 
    void SetExclude(const char *p,const PatternSet *e) { exclude_prefix=p; exclude=e; }
+   void TryRecursive(bool y=true) { try_recursive=y; }
 
    // caller has to delete the resulting FileSet itself.
    FileSet *GetResult() { return result.borrow(); }
+   bool IsRecursive() const { return is_recursive; }
 
    void Need(unsigned mask) { need|=mask; }
    void NoNeed(unsigned mask) { need&=~mask; }
