@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,13 +54,16 @@ ResDecl eta_period   ("xfer:eta-period", "120",ResMgr::UNumberValidate,ResMgr::N
 ResDecl max_redir    ("xfer:max-redirections", "5",ResMgr::UNumberValidate,ResMgr::NoClosure);
 ResDecl buffer_size  ("xfer:buffer-size","0x10000",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
+// It's bad when lftp receives data in small chunks, try to accumulate
+// data in a kernel buffer using a delay and slurp it at once:
 enum {
    // Delays in microseconds
    MAX_DELAY=30000,
    DELAY_STEP=30,
    // This size is related to socket buffer size.
    // When it is too large, tcp slowdown happens.
-   MAX_READ_TO_DELAY=0x4000,
+   // SSL has packet size 0x4000, so we have to use a lower threshold.
+   MAX_READ_TO_DELAY=0x3F00,
 };
 
 // FileCopy
@@ -492,8 +495,12 @@ void FileCopy::LineBuffered(int s)
 
 off_t FileCopy::GetPos() const
 {
-   if(put)
-      return put->GetRealPos() - put->Buffered();
+   if(put) {
+      off_t pos = put->GetRealPos() - put->Buffered();
+      // sometimes Buffered overestimates the amount of buffered data
+      if(pos<0)
+	 pos=0;
+   }
    if(get)
       return get->GetRealPos();
    return 0;
@@ -633,7 +640,7 @@ void FileCopy::Kill(int sig)
    if(put) put->Kill(sig);
 }
 
-SMTaskRef<Log> FileCopy::transfer_log;
+Ref<Log> FileCopy::transfer_log;
 
 void FileCopy::LogTransfer()
 {
@@ -876,7 +883,8 @@ int FileCopyPeerFA::Do()
 	 if(suggested_filename && auto_rename)
 	 {
 	    const char *new_name=dir_file(dirname(file),suggested_filename);
-	    session->Rename(file,new_name);
+	    bool clobber=temp_file;
+	    session->Rename(file,new_name,clobber);
 	    return MOVED;
 	 }
 	 done=true;
