@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,7 @@ protected:
    bool do_verify;
    bool removing;
    bool file_removed;
+   bool temp_file;
 
    bool done;
 
@@ -85,7 +86,7 @@ public:
 
    void SetDate(time_t d,int p=0);
    void SetSize(off_t s);
-   void SetEntitySize(off_t s) { e_size=s; }
+   void SetEntitySize(off_t s) { if(!ascii) e_size=s; }
 
    void DontCopyDate() { do_set_date=false; }
    void DontVerify() { do_verify=false; }
@@ -140,12 +141,9 @@ public:
    virtual const Ref<FDStream>& GetLocal() const { return Ref<FDStream>::null; }
 
    const char *GetSuggestedFileName() { return suggested_filename; }
-   void SetSuggestedFileName(const char *f)
-      {
-	 if(f && !suggested_filename)
-	    suggested_filename.set(f);
-      }
+   void SetSuggestedFileName(const char *f) { if(f) suggested_filename.set(f); }
    void AutoRename(bool yes=true) { auto_rename=yes; }
+   const char *UseTempFile(const char *);
 };
 
 class FileCopy : public SMTask
@@ -256,7 +254,7 @@ public:
    bool WritePending() { return put && put->WritePending(); }
 
    void LogTransfer();
-   static SMTaskRef<Log> transfer_log;
+   static Ref<Log> transfer_log;
 };
 
 class FileVerificator : public SMTask
@@ -362,6 +360,7 @@ class FileCopyPeerFDStream : public FileCopyPeer
 
    bool create_fg_data;
    bool need_seek;
+   bool close_when_done;
 
    SMTaskRef<FileVerificator> verify;
 
@@ -378,6 +377,7 @@ public:
 
    void DontCreateFgData() { create_fg_data=false; }
    void NeedSeek() { need_seek=true; }
+   void CloseWhenDone() { close_when_done=true; }
    void WantSize();
    void RemoveFile();
    void SetBase(off_t b) { seek_base=b; }
