@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FtpListInfo.cc,v 1.66 2011/06/28 09:45:56 lav Exp $ */
-
 #include <config.h>
 #include "FtpListInfo.h"
 #include "FileSet.h"
@@ -265,8 +263,8 @@ FileInfo *ParseFtpLongList_NT(char *line,int *err,const char *tz)
    if(t==0)
       ERR;
    int hour,minute;
-   char am;
-   if(sscanf(t,"%2d:%2d%c",&hour,&minute,&am)!=3)
+   char am='A'; // AM/PM is optional
+   if(sscanf(t,"%2d:%2d%c",&hour,&minute,&am)<2)
       ERR;
    t = NEXT_TOKEN;
    if(t==0)
@@ -727,10 +725,13 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
    bool type_known=false;
    int perms=-1;
 
-   /* NcFTPd does not put a semicolon after last fact, workaround it. */
-   if(!strstr(line,"; "))
-   {
-      char *space=strchr(line,' ');
+   char *space=strstr(line,"; ");
+   if(space) {
+      name=space+2;
+      *space=0;
+   } else {
+      /* NcFTPd does not put a semicolon after last fact, workaround it. */
+      space=strchr(line,' ');
       if(!space)
 	 ERR;
       name=space+1;
@@ -739,11 +740,6 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
 
    for(char *tok=strtok(line,";"); tok; tok=strtok(0,";"))
    {
-      if(tok[0]==' ')
-      {
-	 name=tok+1;
-	 break;
-      }
       if(!strcasecmp(tok,"Type=cdir")
       || !strcasecmp(tok,"Type=pdir")
       || !strcasecmp(tok,"Type=dir"))
