@@ -112,6 +112,7 @@ enum {
 #define H_REQUESTED_RANGE_NOT_SATISFIABLE(x) ((x) == H_Requested_Range_Not_Satisfiable)
 #define H_TRANSIENT(x)	((x)==H_Request_Timeout || (x)==H_Bad_Gateway || (x)==H_Service_Unavailable || (x)==H_Gateway_Timeout)
 #define H_UNSUPPORTED(x) ((x)==H_Bad_Request || (x)==H_Not_Implemented)
+#define H_AUTH_REQ(x)	((x)==H_Unauthorized || (x)==H_Proxy_Authentication_Required)
 
 #ifndef EINPROGRESS
 #define EINPROGRESS -1
@@ -158,9 +159,11 @@ void Http::Init()
    array_send=0;
 
    chunked=false;
+   chunked_trailer=false;
    chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
-   chunked_trailer=false;
+
+   request_pos=0;
 
    no_ranges=false;
    seen_ranges_bytes=false;
@@ -168,10 +171,8 @@ void Http::Init()
    no_cache_this=false;
    no_cache=false;
 
-   auth_sent=0;
-   proxy_auth_sent=0;
-   auth_scheme=HttpAuth::NONE;
-   proxy_auth_scheme=HttpAuth::NONE;
+   auth_sent[0]=auth_sent[1]=0;
+   auth_scheme[0]=auth_scheme[1]=HttpAuth::NONE;
 
    use_propfind_now=true;
    allprop="";
@@ -227,14 +228,20 @@ void Http::DisconnectLL()
       LogNote(7,_("Closing HTTP connection"));
       conn=0;
    }
-   if((mode==STORE && state!=DONE && real_pos>0)
-   && !Error())
-   {
+
+   if(!Error() && !H_AUTH_REQ(status_code))
+      auth_sent[0]=auth_sent[1]=0;
+
+   if(state!=DONE && (real_pos>0 || special==HTTP_POST)
+   && !Error() && !H_AUTH_REQ(status_code)) {
       if(last_method && !strcmp(last_method,"POST"))
 	 SetError(FATAL,_("POST method failed"));
-      else
+      else if(mode==STORE)
 	 SetError(STORE_FAILED,0);
    }
+   if(mode==STORE && H_AUTH_REQ(status_code))
+      pos=real_pos=request_pos; // resend all the data again
+
    last_method=0;
    last_uri.unset();
    last_url.unset();
@@ -255,9 +262,10 @@ void Http::ResetRequestData()
    keep_alive_max=-1;
    array_send=fileset_for_info?fileset_for_info->curr_index():0;
    chunked=false;
+   chunked_trailer=false;
    chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
-   chunked_trailer=false;
+   request_pos=0;
    propfind=0;
    inflate=0;
    seen_ranges_bytes=false;
@@ -298,10 +306,8 @@ void Http::Close()
    }
    array_send=0;
    no_cache_this=false;
-   auth_sent=0;
-   proxy_auth_sent=0;
-   auth_scheme=HttpAuth::NONE;
-   proxy_auth_scheme=HttpAuth::NONE;
+   auth_sent[0]=auth_sent[1]=0;
+   auth_scheme[0]=auth_scheme[1]=HttpAuth::NONE;
    no_ranges=false;
    use_propfind_now=QueryBool("use-propfind",hostname);
    special=HTTP_NONE;
@@ -440,31 +446,30 @@ void Http::SendBasicAuth(const char *tag,const char *user,const char *pass)
    SendBasicAuth(tag,xstring::cat(user,":",pass,NULL));
 }
 
+void Http::SendAuth(HttpAuth::target_t target,const char *user,const char *uri)
+{
+   auth_scheme[target]=HttpAuth::NONE;
+   if(!user)
+      return;
+   HttpAuth *auth=HttpAuth::Get(target,GetFileURL(file,NO_USER),user);
+   if(auth && auth->Update(last_method,uri)) {
+      auth_sent[target]++;
+      Send(auth->GetHeader());
+   }
+}
+
 void Http::SendProxyAuth()
 {
-   proxy_auth_scheme=HttpAuth::NONE;
-   if(!proxy || !proxy_user || !proxy_pass)
-      return;
-   HttpAuth *auth=HttpAuth::Get(HttpAuth::PROXY,GetFileURL(file,NO_USER),proxy_user);
-   if(auth && auth->Update(last_method,last_url)) {
-      proxy_auth_sent++;
-      Send(auth->GetHeader());
-      return;
-   }
+   SendAuth(HttpAuth::PROXY,proxy_user,last_url);
 }
 
 void Http::SendAuth()
 {
-   if(!auth_scheme && hftp && user && pass && QueryBool("use-authorization",proxy)) {
+   if(hftp && !auth_scheme[HttpAuth::WWW] && user && pass && QueryBool("use-authorization",proxy)) {
       SendBasicAuth("Authorization",user,pass);
       return;
    }
-   auth_scheme=HttpAuth::NONE;
-   HttpAuth *auth=HttpAuth::Get(HttpAuth::WWW,GetFileURL(file,NO_USER),user);
-   if(auth && auth->Update(last_method,last_uri,0)) {
-      auth_sent++;
-      Send(auth->GetHeader());
-   }
+   SendAuth(HttpAuth::WWW,user,last_uri);
 }
 void Http::SendCacheControl()
 {
@@ -724,6 +729,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	 Send("Content-length: %lld\r\n",(long long)(entity_size-pos));
       if(pos>0 && entity_size<0)
       {
+	 request_pos=pos;
 	 if(limit==FILE_END)
 	    Send("Range: bytes=%lld-\r\n",(long long)pos);
 	 else
@@ -731,6 +737,7 @@ void Http::SendRequest(const char *connection,const char *f)
       }
       else if(pos>0)
       {
+	 request_pos=pos;
 	 Send("Range: bytes=%lld-%lld/%lld\r\n",(long long)pos,
 		     (long long)((limit==FILE_END || limit>entity_size ? entity_size : limit)-1),
 		     (long long)entity_size);
@@ -829,9 +836,10 @@ void Http::SendRequest(const char *connection,const char *f)
 
    keep_alive=false;
    chunked=false;
+   chunked_trailer=false;
    chunk_size=CHUNK_SIZE_UNKNOWN;
    chunk_pos=0;
-   chunked_trailer=false;
+   request_pos=0;
    inflate=0;
    no_ranges=false;
 
@@ -910,6 +918,27 @@ void Http::ProceedArrayInfo()
    }
 }
 
+void Http::NewAuth(const char *hdr,HttpAuth::target_t target,const char *user,const char *pass)
+{
+   if(!user || !pass)
+      return;
+
+   // FIXME: keep a request queue, get the URI from the queue.
+   const char *uri=GetFileURL(file,NO_USER);
+
+   Ref<HttpAuth::Challenge> chal(new HttpAuth::Challenge(hdr));
+   bool stale=chal->GetParam("stale").eq_nc("true");
+
+   if(auth_sent[target]>(stale?1:0))
+      return;
+
+   HttpAuth::scheme_t new_scheme=chal->GetSchemeCode();
+   if(new_scheme<=auth_scheme[target])
+      return;
+   if(HttpAuth::New(target,uri,chal.borrow(),user,pass))
+      auth_scheme[target]=new_scheme;
+}
+
 void Http::HandleHeaderLine(const char *name,const char *value)
 {
    // use a perfect hash
@@ -1010,9 +1039,9 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(!strcasecmp(value,"chunked"))
       {
 	 chunked=true;
+	 chunked_trailer=false;
 	 chunk_size=CHUNK_SIZE_UNKNOWN;	  // expecting first chunk
 	 chunk_pos=0;
-	 chunked_trailer=false;
       }
       return;
 
@@ -1051,6 +1080,8 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       return;
    }
    case_hh("WWW-Authenticate",'W') {
+      if(status_code!=H_Unauthorized)
+	 return;
       const char *user=this->user;
       const char *pass=this->pass;
       if(!user || !pass) {
@@ -1066,30 +1097,13 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	    }
 	 }
       }
-      if(user && pass) {
-	 // FIXME: keep a request queue, get the URI from the queue.
-	 const char *uri=GetFileURL(file,NO_USER);
-
-	 Ref<HttpAuth::Challenge> chal(new HttpAuth::Challenge(value));
-	 bool stale=chal->GetParam("stale").eq_nc("true");
-	 if(status_code==H_Unauthorized) {
-	    if(auth_sent>(stale?1:0))
-	       return;
-	    HttpAuth::scheme_t new_scheme=chal->GetSchemeCode();
-	    if(new_scheme<=auth_scheme)
-	       return;
-	    if(HttpAuth::New(HttpAuth::WWW,uri,chal.borrow(),user,pass))
-	       auth_scheme=new_scheme;
-	 } else if(status_code==H_Proxy_Authentication_Required) {
-	    if(proxy_auth_sent>(stale?1:0))
-	       return;
-	    HttpAuth::scheme_t new_scheme=chal->GetSchemeCode();
-	    if(new_scheme<=proxy_auth_scheme)
-	       return;
-	    if(HttpAuth::New(HttpAuth::PROXY,uri,chal.borrow(),proxy_user,proxy_pass))
-	       proxy_auth_scheme=new_scheme;
-	 }
-      }
+      NewAuth(value,HttpAuth::WWW,user,pass);
+      return;
+   }
+   case_hh("Proxy-Authenticate",'P') {
+      if(status_code!=H_Proxy_Authentication_Required)
+	 return;
+      NewAuth(value,HttpAuth::PROXY,proxy_user,proxy_pass);
       return;
    }
    default:
@@ -1282,7 +1296,7 @@ int Http::Do()
 		  file_url.append('/');
 	       file_url.append_url_encoded(cwd,URL_PATH_UNSAFE);
 	    }
-	    if(*scan!='/')
+	    if(*scan!='/' && file_url.last_char()!='/')
 	       file_url.append('/');
 	    file_url.append(scan);
 	    file_url.truncate_at(' ');
@@ -1704,9 +1718,10 @@ int Http::Do()
 	 return MOVED;
       }
 
-      if((status_code==H_Unauthorized && auth_scheme)
-      || (status_code==H_Proxy_Authentication_Required && proxy_auth_scheme)) {
+      if((status_code==H_Unauthorized && auth_scheme[HttpAuth::WWW])
+      || (status_code==H_Proxy_Authentication_Required && auth_scheme[HttpAuth::PROXY])) {
 	 // retry with authentication
+	 retries--;
 	 state=RECEIVING_BODY;
 	 LogErrorText();
 	 Disconnect();
