@@ -144,9 +144,11 @@ class Http : public NetAccess
    int array_send;
 
    bool chunked;
+   bool chunked_trailer;
    long chunk_size;
    off_t chunk_pos;
-   bool chunked_trailer;
+
+   off_t request_pos;
 
    Ref<DirectedBuffer> inflate;
    SMTaskRef<IOBuffer> propfind;
@@ -161,10 +163,11 @@ class Http : public NetAccess
    bool no_cache;
    bool no_cache_this;
 
-   int  auth_sent;
-   int	proxy_auth_sent;
-   HttpAuth::scheme_t auth_scheme;
-   HttpAuth::scheme_t proxy_auth_scheme;
+   // for WWW[0] and PROXY[1]
+   int  auth_sent[2];
+   HttpAuth::scheme_t auth_scheme[2];
+   void NewAuth(const char *hdr,HttpAuth::target_t target,const char *user,const char *pass);
+   void SendAuth(HttpAuth::target_t target,const char *user,const char *uri);
 
    bool use_propfind_now;
    const char *allprop;
