@@ -299,8 +299,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    if((flags&CONTINUE)
 	    && old->Has(file->TYPE) && old->filetype==old->NORMAL
 	    && (flags&IGNORE_TIME ||
-	    	(file->Has(file->DATE) && old->Has(old->DATE)
-	    	&& file->date + file->date.ts_prec < old->date - old->date.ts_prec))
+		(file->Has(file->DATE) && old->Has(old->DATE)
+		&& file->date + file->date.ts_prec < old->date - old->date.ts_prec))
 	    && file->Has(file->SIZE) && old->Has(old->SIZE)
 	    && file->size >= old->size)
 	    {
@@ -386,7 +386,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 CopyJob *cp=(use_pget ? new pgetJob(c,file->name,pget_n) : new CopyJob(c,file->name,"mirror"));
 	 if(file->Has(file->DATE))
 	    cp->SetDate(file->date);
-	 if(file->Has(file->SIZE))
+	 if(file->Has(file->SIZE) && !FlagSet(IGNORE_SIZE))
 	    cp->SetSize(file->size);
 	 TransferStarted(cp);
 	 cp->cmdline.vset("\\transfer `",source_name_rel,"'",NULL);
@@ -461,6 +461,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->pget_n=pget_n;
 	 mj->pget_minchunk=pget_minchunk;
 	 mj->remove_source_files=remove_source_files;
+	 mj->remove_source_dirs=remove_source_dirs;
 	 mj->skip_noaccess=skip_noaccess;
 	 mj->create_target_dir=create_target_dir;
 	 mj->no_target_dir=no_target_dir;
@@ -513,10 +514,12 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    }
 	    else
 	       stats.new_symlinks++;
+	    Report(_("Making symbolic link `%s' to `%s'"),target_name_rel,file->symlink.get());
 	    mvJob *j=new mvJob(target_session->Clone(),file->symlink,target_name,FA::SYMLINK);
 	    if(remove_target)
 	       j->RemoveTargetFirst();
 	    AddWaiting(j);
+	    RemoveSourceLater(file);
 	    break;
 	 }
 
@@ -533,33 +536,31 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       goto skip;
 	 }
 
-	 if(file->Has(file->SYMLINK))
+	 struct stat st;
+	 if(lstat(target_name,&st)!=-1)
 	 {
-	    struct stat st;
-	    if(lstat(target_name,&st)!=-1)
+	    Report(_("Removing old local file `%s'"),target_name_rel);
+	    stats.mod_symlinks++;
+	    if(remove(target_name)==-1)
 	    {
-	       Report(_("Removing old local file `%s'"),target_name_rel);
-	       stats.mod_symlinks++;
-	       if(remove(target_name)==-1)
-	       {
-		  eprintf("mirror: remove(%s): %s\n",target_name,strerror(errno));
-		  goto skip;
-	       }
+	       eprintf("mirror: remove(%s): %s\n",target_name,strerror(errno));
+	       goto skip;
 	    }
-	    else
-	    {
-	       if(flags&ONLY_EXISTING)
-	       {
-		  Report(_("Skipping symlink `%s' (only-existing)"),target_name_rel);
-		  goto skip;
-	       }
-	       stats.new_symlinks++;
-	    }
-	    Report(_("Making symbolic link `%s' to `%s'"),target_name_rel,file->symlink.get());
-	    res=symlink(file->symlink,target_name);
-	    if(res==-1)
-	       eprintf("mirror: symlink(%s): %s\n",target_name,strerror(errno));
 	 }
+	 else
+	 {
+	    if(flags&ONLY_EXISTING)
+	    {
+	       Report(_("Skipping symlink `%s' (only-existing)"),target_name_rel);
+	       goto skip;
+	    }
+	    stats.new_symlinks++;
+	 }
+	 Report(_("Making symbolic link `%s' to `%s'"),target_name_rel,file->symlink.get());
+	 res=symlink(file->symlink,target_name);
+	 if(res==-1)
+	    eprintf("mirror: symlink(%s): %s\n",target_name,strerror(errno));
+	 RemoveSourceLater(file);
 	 break;
       }
    case FileInfo::UNKNOWN:
@@ -1007,7 +1008,7 @@ int   MirrorJob::Do()
       while(transfer_count<parallel && state==WAITING_FOR_TRANSFER)
       {
 	 file=to_transfer->curr();
-      	 if(!file)
+	 if(!file)
 	 {
 	    // go to the next step only when all transfers have finished
 	    if(waiting_num>0)
@@ -1198,7 +1199,7 @@ int   MirrorJob::Do()
 	 if(FlagSet(ALLOW_CHOWN) && same)
 	    same->LocalChown(target_dir);
       }
-      if(remove_source_files && same)
+      if(remove_source_files && (same || to_rm_src))
 	 goto pre_SOURCE_REMOVING_SAME;
    pre_FINISHING:
       set_state(FINISHING);
@@ -1213,6 +1214,35 @@ int   MirrorJob::Do()
       if(waiting_num>0)
 	 break;
 
+      // all jobs finished.
+      if(remove_source_dirs) {
+	 // remove source directory once.
+	 remove_source_dirs=false;
+	 if(script)
+	 {
+	    ArgV args("rmdir");
+	    args.Append(source_session->GetFileURL(source_dir));
+	    xstring_ca cmd(args.CombineQuoted());
+	    fprintf(script,"%s\n",cmd.get());
+	 }
+	 if(!script_only)
+	 {
+	    ArgV *args=new ArgV("rmdir");
+	    args->Append(source_dir);
+	    args->seek(1);
+	    rmJob *j=new rmJob(source_session->Clone(),args);
+	    j->cmdline.set_allocated(args->Combine());
+	    j->Rmdir();
+	    JobStarted(j);
+	 }
+	 if(source_relative_dir)
+	    Report(_("Removing source directory `%s'"),source_relative_dir.get());
+	 m=MOVED;
+	 break;
+      }
+
+      // all jobs finished and src dir removed, if needed.
+
       transfer_count++; // parent mirror will decrement it.
       if(parent_mirror)
 	 parent_mirror->stats.Add(stats);
@@ -1231,6 +1261,10 @@ int   MirrorJob::Do()
       goto pre_DONE;
 
    pre_SOURCE_REMOVING_SAME:
+      if(!same)
+	 same=to_rm_src.borrow();
+      else if(to_rm_src)
+	 same->Merge(to_rm_src);
       same->rewind();
       set_state(SOURCE_REMOVING_SAME);
       m=MOVED;
@@ -1342,6 +1376,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
 
    use_cache=false;
    remove_source_files=false;
+   remove_source_dirs=false;
    skip_noaccess=false;
 
    parallel=1;
@@ -1568,6 +1603,30 @@ const char *MirrorJob::AddPattern(Ref<PatternSet>& exclude,char opt,const char *
    return NULL; // no error
 }
 
+const char *MirrorJob::AddPatternsFrom(Ref<PatternSet>& exclude,char opt,const char *file)
+{
+   FILE *f=fopen(file,"r");
+   if(!f)
+      return xstring::format("%s: %s",file,strerror(errno));
+
+   xstring line;
+   const char *err=0;
+   int c;
+   while(!feof(f)) {
+      line.truncate();
+      while((c=getc(f))!=EOF && c!='\n')
+	 line.append(c);
+      if(line.length()>0) {
+	 err=AddPattern(exclude,opt,line);
+	 if(err)
+	    break;
+      }
+   }
+
+   fclose(f);
+   return err;
+}
+
 const char *MirrorJob::SetRecursionMode(const char *m)
 {
    struct { const char name[8]; recursion_mode_t mode; } map[]={
@@ -1608,6 +1667,7 @@ CMD(mirror)
       OPT_ONLY_EXISTING,
       OPT_PERMS,
       OPT_REMOVE_SOURCE_FILES,
+      OPT_REMOVE_SOURCE_DIRS,
       OPT_SCRIPT,
       OPT_SCRIPT_ONLY,
       OPT_SIZE_RANGE,
@@ -1634,6 +1694,10 @@ CMD(mirror)
       {"exclude",required_argument,0,'x'},
       {"include-glob",required_argument,0,'I'},
       {"exclude-glob",required_argument,0,'X'},
+      {"include-rx-from",required_argument,0,'i'+'f'},
+      {"exclude-rx-from",required_argument,0,'x'+'f'},
+      {"include-glob-from",required_argument,0,'I'+'f'},
+      {"exclude-glob-from",required_argument,0,'X'+'f'},
       {"only-newer",no_argument,0,'n'},
       {"no-recursion",no_argument,0,'r'},
       {"no-perms",no_argument,0,'p'},
@@ -1650,6 +1714,8 @@ CMD(mirror)
       {"no-dereference",no_argument,0,OPT_NO_DEREFERENCE},
       {"use-cache",no_argument,0,OPT_USE_CACHE},
       {"Remove-source-files",no_argument,0,OPT_REMOVE_SOURCE_FILES},
+      {"Remove-source-dirs",no_argument,0,OPT_REMOVE_SOURCE_DIRS},
+      {"Move",no_argument,0,OPT_REMOVE_SOURCE_DIRS},
       {"parallel",optional_argument,0,'P'},
       {"ignore-time",no_argument,0,OPT_IGNORE_TIME},
       {"ignore-size",no_argument,0,OPT_IGNORE_SIZE},
@@ -1694,6 +1760,7 @@ CMD(mirror)
    const char *older_than=0;
    Ref<Range> size_range;
    bool  remove_source_files=false;
+   bool  remove_source_dirs=false;
    bool	 skip_noaccess=ResMgr::QueryBool("mirror:skip-noaccess",0);
    int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
@@ -1756,8 +1823,20 @@ CMD(mirror)
 	 const char *err=MirrorJob::AddPattern(exclude,opt,optarg);
 	 if(err)
 	 {
-	    eprintf(_("%s: regular expression `%s': %s\n"),
-	       args->a0(),optarg,err);
+	    eprintf("%s: %s\n",args->a0(),err);
+	    goto no_job;
+	 }
+	 break;
+      }
+      case('x'+'f'):
+      case('i'+'f'):
+      case('X'+'f'):
+      case('I'+'f'):
+      {
+	 const char *err=MirrorJob::AddPatternsFrom(exclude,opt-'f',optarg);
+	 if(err)
+	 {
+	    eprintf("%s: %s\n",args->a0(),err);
 	    goto no_job;
 	 }
 	 break;
@@ -1815,6 +1894,9 @@ CMD(mirror)
       case(OPT_REMOVE_SOURCE_FILES):
 	 remove_source_files=true;
 	 break;
+      case(OPT_REMOVE_SOURCE_DIRS):
+	 remove_source_dirs=true;
+	 break;
       case(OPT_IGNORE_TIME):
 	 flags|=MirrorJob::IGNORE_TIME;
 	 break;
@@ -2020,6 +2102,8 @@ CMD(mirror)
    j->UseCache(use_cache);
    if(remove_source_files)
       j->RemoveSourceFiles();
+   if(remove_source_dirs)
+      j->RemoveSourceDirs();
    if(skip_noaccess)
       j->SkipNoAccess();
    if(parallel<0)
