@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -418,6 +418,12 @@ int DirectedBuffer::MoveDataHere(Buffer *buf,int size)
       return Buffer::MoveDataHere(buf,size);
    return size;
 }
+void DirectedBuffer::PutEOF()
+{
+   if(mode==PUT && translator)
+      translator->PutTranslated(this,0,0);
+   Buffer::PutEOF();
+}
 
 void DirectedBuffer::EmbraceNewData(int len)
 {
@@ -496,6 +502,8 @@ int IOBuffer::Do()
 	 RateAdd(res);
 	 buffer_ptr+=res;
 	 event_time=now;
+	 if(eof)
+	    PutEOF_LL();
 	 return MOVED;
       }
       break;
