@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -106,6 +106,8 @@ CMD(false);
 # define cmd_torrent 0
 #endif
 
+enum { DEFAULT_DEBUG_LEVEL=9 };
+
 const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 {
    {"!",       cmd_shell,  N_("!<shell-command>"),
@@ -285,12 +287,15 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("`lftp' is the first command executed by lftp after rc files\n"
 	 " -f <file>           execute commands from the file and exit\n"
 	 " -c <cmd>            execute the commands and exit\n"
+	 " --norc              don't execute rc files from the home directory\n"
 	 " --help              print this help and exit\n"
 	 " --version           print lftp version and exit\n"
-	 "Other options are the same as in `open' command\n"
+	 "Other options are the same as in `open' command:\n"
 	 " -e <cmd>            execute the command just after selecting\n"
 	 " -u <user>[,<pass>]  use the user/password for authentication\n"
 	 " -p <port>           use the port for connection\n"
+	 " -s <slot>           assign the connection to this slot\n"
+	 " -d                  switch on debugging mode\n"
 	 " <site>              host name, URL or bookmark name\n")},
    {"ln",      cmd_ln,	    N_("ln [-s] <file1> <file2>"),
 	 N_("Link <file1> to <file2>\n")},
@@ -345,6 +350,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -u <user>[,<pass>]  use the user/password for authentication\n"
 	 " -p <port>           use the port for connection\n"
 	 " -s <slot>           assign the connection to this slot\n"
+	 " -d                  switch on debugging mode\n"
 	 " <site>              host name, URL or bookmark name\n")},
    {"pget",    cmd_get,    N_("pget [OPTS] <rfile> [-o <lfile>]"),
 	 N_("Gets the specified file using several connections. This can speed up transfer,\n"
@@ -423,7 +429,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "exact application of the setting. See lftp(1) for details.\n"
          "If set is called with no variable then only altered settings are listed.\n"
 	 "It can be changed by options:\n"
-   	 " -a  list all settings, including default values\n"
+	 " -a  list all settings, including default values\n"
 	 " -d  list only default values, not necessary current ones\n")},
    {"shell", ALIAS_FOR2("!",shell)},
    {"site",    cmd_ls,	   N_("site <site-cmd>"),
@@ -432,7 +438,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"sleep",   cmd_sleep, 0, HELP_IN_MODULE},
    {"slot",    cmd_slot, 0,
         N_("Usage: slot [<label>]\n"
-   	"List assigned slots.\n"
+	"List assigned slots.\n"
 	"If <label> is specified, switch to the slot named <label>.\n")},
    {"source",  cmd_source, N_("source <file>"),
 	 N_("Execute commands recorded in file <file>\n")},
@@ -727,16 +733,27 @@ void CmdExec::Exit(int code)
    return;
 }
 
+void CmdExec::enable_debug(const char *opt)
+{
+   int level=DEFAULT_DEBUG_LEVEL;
+   if(opt && isdigit((unsigned char)opt[0]))
+      level=atoi(opt);
+   Log::global->Enable();
+   Log::global->SetLevel(level);
+   Log::global->SetOutput(2,false);
+   Log::global->ShowNothing();
+}
+
 CmdFeeder *lftp_feeder=0;
 Job *CmdExec::builtin_lftp()
 {
    int c;
    xstring cmd;
-   bool debug=false;
    static struct option lftp_options[]=
    {
       {"help",no_argument,0,'h'},
       {"version",no_argument,0,'v'},
+      {"debug",optional_argument,0,'d'},
       {0,0,0,0}
    };
 
@@ -761,18 +778,14 @@ Job *CmdExec::builtin_lftp()
 	 cmd.append("\n\n");
 	 break;
       case('d'):
-	 debug=true;
+	 enable_debug(optarg);
 	 break;
       }
    }
    opterr=true;
 
    if(cmd)
-   {
       PrependCmd(cmd);
-      if(debug)
-	 PrependCmd("debug;");
-   }
 
    if(Done() && lftp_feeder)  // no feeder and no commands
    {
@@ -795,7 +808,6 @@ Job *CmdExec::builtin_open()
 {
    ReuseSavedSession();
 
-   bool	 debug=false;
    const char *port=NULL;
    const char *host=NULL;
    const char *path=NULL;
@@ -812,6 +824,7 @@ Job *CmdExec::builtin_open()
       OPT_USER,
       OPT_PASSWORD,
       OPT_ENV_PASSWORD,
+      OPT_NORC,
    };
    static struct option open_options[]=
    {
@@ -824,6 +837,7 @@ Job *CmdExec::builtin_open()
       {"no-bookmark",no_argument,0,'B'},
       {"slot",required_argument,0,'s'},
       {"help",no_argument,0,'h'},
+      {"norc",no_argument,0,OPT_NORC},
       {0,0,0,0}
    };
 
@@ -832,7 +846,7 @@ Job *CmdExec::builtin_open()
       switch(c)
       {
       case('s'):
-      	if (*optarg) ChangeSlot(optarg);
+	if (*optarg) ChangeSlot(optarg);
         break;
       case('p'):
 	 port=optarg;
@@ -846,7 +860,7 @@ Job *CmdExec::builtin_open()
 	 if(sep==NULL)
 	    sep=strchr(optarg,':');
 	 if(sep==NULL)
-   	    break;
+	    break;
 	 *sep=0;
 	 pass=sep+1;
 	 insecure=true;
@@ -862,7 +876,7 @@ Job *CmdExec::builtin_open()
 	 pass=getenv("LFTP_PASSWORD");
 	 break;
       case('d'):
-	 debug=true;
+	 enable_debug(optarg);
 	 break;
       case('e'):
 	 cmd_to_exec=optarg;
@@ -870,6 +884,11 @@ Job *CmdExec::builtin_open()
       case('B'):
 	 no_bm=true;
 	 break;
+      case(OPT_NORC):
+	 if(!strcmp(op,"lftp"))
+	    break;
+	 eprintf(_("%s: unrecognized option '%s'\n"),op,"--norc");
+	 goto help;
       case('h'):
 	 if(!strcmp(op,"lftp"))
 	 {
@@ -877,6 +896,7 @@ Job *CmdExec::builtin_open()
 	    return 0;
 	 }
       case('?'):
+      help:
 	 if(!strcmp(op,"lftp"))
 	    eprintf(_("Try `%s --help' for more information\n"),op);
 	 else
@@ -1059,9 +1079,6 @@ Job *CmdExec::builtin_open()
       PrependCmd(s);
    }
 
-   if(debug)
-      PrependCmd("debug\n");
-
    if(slot)
       ConnectionSlot::Set(slot,session);
 
@@ -1637,7 +1654,7 @@ CMD(get)
 {
    int opt;
    bool cont=false;
-   const char *opts="+cEeuaO:";
+   const char *opts="+cEeuaO:q";
    const char *op=args->a0();
    ArgV	 *get_args=new ArgV(op);
    int n_conn=1;
@@ -1647,16 +1664,17 @@ CMD(get)
    bool glob=false;
    bool make_dirs=false;
    bool reverse=false;
+   bool quiet=false;
    xstring_c output_dir;
 
    if(!strncmp(op,"re",2))
    {
       cont=true;
-      opts="+EuaO:";
+      opts="+EuaO:q";
    }
    if(!strcmp(op,"pget"))
    {
-      opts="+n:ceuO:";
+      opts="+n:ceuO:q";
       n_conn=0; // default, which means to take pget:default-n
    }
    else if(!strcmp(op,"put") || !strcmp(op,"reput"))
@@ -1666,12 +1684,12 @@ CMD(get)
    else if(!strcmp(op,"mget"))
    {
       glob=true;
-      opts="cEeadO:";
+      opts="cEeadO:q";
    }
    else if(!strcmp(op,"mput"))
    {
       glob=true;
-      opts="cEeadO:";
+      opts="cEeadO:q";
       reverse=true;
    }
    if(!reverse)
@@ -1710,6 +1728,9 @@ CMD(get)
       case('O'):
 	 output_dir.set(optarg);
 	 break;
+      case('q'):
+	 quiet=true;
+	 break;
       case('?'):
       err:
 	 eprintf(_("Try `help %s' for more information.\n"),op);
@@ -1739,6 +1760,7 @@ CMD(get)
 	 j->Ascii();
       if(output_dir)
 	 j->OutputDir(output_dir.borrow());
+      j->Quiet(quiet);
       return j;
    }
    args->back();
@@ -1773,6 +1795,7 @@ CMD(get)
       j->Reverse();
    if(n_conn!=1)
       j->SetCopyJobCreator(new pCopyJobCreator(n_conn));
+   j->Quiet(quiet);
    return j;
 }
 
@@ -2042,30 +2065,25 @@ CMD(exit)
 CMD(debug)
 {
    const char *op=args->a0();
-   int	 new_dlevel=9;
+   int	 new_dlevel=DEFAULT_DEBUG_LEVEL;
    char	 *debug_file_name=0;
    int 	 fd=-1;
    bool  enabled=true;
    bool	 show_pid=false;
    bool	 show_time=false;
    bool	 show_context=false;
+   int	 trunc=0;
 
    int opt;
-   while((opt=args->getopt("o:ptc"))!=EOF)
+   while((opt=args->getopt("To:ptc"))!=EOF)
    {
       switch(opt)
       {
+      case('T'):
+	 trunc=O_TRUNC;
+	 break;
       case('o'):
 	 debug_file_name=optarg;
-	 if(fd!=-1)
-	    close(fd);
-	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK,0600);
-	 if(fd==-1)
-	 {
-	    perror(debug_file_name);
-	    return 0;
-	 }
-	 fcntl(fd,F_SETFD,FD_CLOEXEC);
 	 break;
       case 'p':
 	 show_pid=true;
@@ -2082,6 +2100,16 @@ CMD(debug)
       }
    }
 
+   if(debug_file_name) {
+      fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK|trunc,0600);
+      if(fd==-1)
+      {
+	 perror(debug_file_name);
+	 return 0;
+      }
+      fcntl(fd,F_SETFD,FD_CLOEXEC);
+   }
+
    if(fd==-1)
       Log::global->SetOutput(2,false);
    else
@@ -2241,7 +2269,7 @@ CMD(kill)
       {
 	 eprintf(_("%s: %s - not a number\n"),op,arg);
 	 exit_code=1;
-      	 continue;
+	 continue;
       }
       n=atoi(arg);
       if(Job::Running(n))
@@ -2652,7 +2680,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2015);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2016);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -2826,10 +2854,7 @@ CMD(bookmark)
 	 {
 	    value=session->GetConnectURL(flags);
 	    // encode some more characters, special to CmdExec parser.
-	    xstring& a=url::encode(value,"&;|\"'\\");
-	    if(value[0] && last_char(value)!='/')
-	       a.append('/');
-	    value=a;
+	    value=url::encode(value,"&;|\"'\\");
 	 }
 	 if(value==0 || value[0]==0)
 	    value="\"\"";
@@ -2839,7 +2864,7 @@ CMD(bookmark)
 	    return 0;
 	 }
 	 lftp_bookmarks.Add(key,value);
-   	 exit_code=0;
+	 exit_code=0;
       }
    }
    else if(!strcasecmp(op,"delete"))
@@ -3264,6 +3289,7 @@ CMD(get1)
       {"output",required_argument,0,'o'},
       {"remove-source-later",no_argument,0,'E'},
       {"remove-target-first",no_argument,0,'e'},
+      {"quiet",no_argument,0,'q'},
       {0,0,0,0}
    };
    int opt;
@@ -3271,6 +3297,7 @@ CMD(get1)
    const char *dst=0;
    bool cont=false;
    bool ascii=false;
+   bool quiet=false;
    long long source_region_begin=0,source_region_end=FILE_END;
    long long target_region_begin=0,target_region_end=FILE_END;
    int n,p;
@@ -3306,6 +3333,9 @@ CMD(get1)
 	    goto usage;
 	 }
 	 break;
+      case('q'):
+	 quiet=true;
+	 break;
       case '?':
       usage:
 	 eprintf(_("Usage: %s [OPTS] file\n"),args->a0());
@@ -3378,7 +3408,9 @@ CMD(get1)
    if(ascii)
       c->Ascii();
 
-   return new CopyJob(c,src,args->a0());
+   CopyJob *cj=new CopyJob(c,src,args->a0());
+   cj->Quiet(quiet);
+   return cj;
 }
 
 CMD(slot)
