@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,9 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <pwd.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -42,6 +45,7 @@
 
 #if LIBIDN
 # include <idna.h>
+# include <tld.h>
 #endif
 
 CDECL_BEGIN
@@ -172,7 +176,7 @@ const char *expand_home_relative(const char *s)
 
    if(s[1]==0 || s[1]=='/')
    {
-      home=getenv("HOME");
+      home=get_home();
    }
    else
    {
@@ -881,14 +885,34 @@ const char *memrchr(const char *buf,char c,size_t len)
    return 0;
 }
 
-const xstring& shell_encode(const char *string)
+bool is_shell_special(char c)
+{
+   switch (c)
+   {
+   case '\'':
+   case '(': case ')':
+   case '!': case '{': case '}':		/* reserved words */
+   case '^':
+   case '$': case '`':			/* expansion chars */
+   case '*': case '[': case '?': case ']':	/* globbing chars */
+   case ' ': case '\t': case '\n':		/* IFS white space */
+   case '"': case '\\':		/* quoting chars */
+   case '|': case '&': case ';':		/* shell metacharacters */
+   case '<': case '>':
+   case '#':				/* comment char */
+      return true;
+   }
+   return false;
+}
+
+const xstring& shell_encode(const char *string,int len)
 {
    if(!string)
       return xstring::null;
 
    static xstring result;
 
-   result.get_space(2 + 2 * strlen (string));
+   result.get_space(2 + 2 * len);
    char *r = result.get_non_const();
 
    if(string[0]=='-' || string[0]=='~')
@@ -900,26 +924,9 @@ const xstring& shell_encode(const char *string)
    int c;
    for (const char *s = string; s && (c = *s); s++)
    {
-      switch (c)
-      {
-      case '\'':
-      case '(': case ')':
-      case '!': case '{': case '}':		/* reserved words */
-      case '^':
-      case '$': case '`':			/* expansion chars */
-      case '*': case '[': case '?': case ']':	/* globbing chars */
-      case ' ': case '\t': case '\n':		/* IFS white space */
-      case '"': case '\\':		/* quoting chars */
-      case '|': case '&': case ';':		/* shell metacharacters */
-      case '<': case '>':
-      case '#':				/* comment char */
+      if (is_shell_special(c))
 	 *r++ = '\\';
-	 *r++ = c;
-	 break;
-      default:
-	 *r++ = c;
-	 break;
-      }
+      *r++ = c;
    }
    result.set_length(r-result);
    return (result);
@@ -1014,3 +1021,29 @@ const char *xidna_to_ascii(const char *name)
 #endif//LIBIDN
    return name;
 }
+bool xtld_name_ok(const char *name)
+{
+#if LIBIDN
+   if(mbswidth(name,MBSW_REJECT_INVALID|MBSW_REJECT_UNPRINTABLE)<=0)
+      return false;
+   size_t err_pos;
+   if(tld_check_lz(name,&err_pos,NULL)==TLD_SUCCESS)
+      return true;
+#endif//LIBIDN
+   return false;
+}
+
+bool is_ipv4_address(const char *s)
+{
+   struct in_addr addr;
+   return inet_pton(AF_INET,s,&addr)>0;
+}
+bool is_ipv6_address(const char *s)
+{
+#if INET6
+   struct in6_addr addr;
+   return inet_pton(AF_INET6,s,&addr)>0;
+#else
+   return false;
+#endif
+}
