@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -200,6 +200,7 @@ static ResType lftp_vars[] = {
    {"ftp:catch-size",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:charset",		 "",	  ResMgr::CharsetValidate,0},
    {"ftp:client",		 PACKAGE "/" VERSION,0,0},
+   {"ftp:compressed-re",	 "\\.(bz2|[glrsx7]z|lzma|lz[hox]|[ai]ce|apk|ar[cj]|cab|cfs|dar|[je]ar|lha|isz|pak|rar|sitx?|t(gz|bz2|lz)|tar\\.(gz|xz|bz2|lzma)|war|zipx?|zoo|[arz0][0-9][0-9])$",ResMgr::ERegExpValidate,ResMgr::NoClosure},
    {"ftp:device-prefix",	 "no",	  ResMgr::BoolValidate,0},
    {"ftp:fix-pasv-address",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:ignore-pasv-address",	 "no",	  ResMgr::BoolValidate,0},
@@ -212,11 +213,12 @@ static ResType lftp_vars[] = {
    {"ftp:lang",			 "",	  0,0},
    {"ftp:list-empty-ok",	 "no",	  0,0},
    {"ftp:list-options",		 "",	  0,0},
+   {"ftp:mode-z-level",		 "6",	  ResMgr::UNumberValidate,0},
    {"ftp:nop-interval",		 "120",   ResMgr::UNumberValidate,0},
    {"ftp:passive-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:port-range",		 "full",  ResMgr::RangeValidate,0},
    {"ftp:port-ipv4",		 "",	  ResMgr::IPv4AddrValidate,0},
-   {"ftp:prefer-epsv",		 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:prefer-epsv",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:proxy",		 "",	  FtpProxyValidate,0},
    {"ftp:proxy-auth-type",	 "user",  FtpProxyAuthTypeValidate,0},
    {"ftp:rest-list",		 "no",	  ResMgr::BoolValidate,0},
@@ -250,7 +252,8 @@ static ResType lftp_vars[] = {
    {"ftp:use-hftp",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-mdtm",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-mdtm-overloaded",	 "no",	  ResMgr::BoolValidate,0},
-   {"ftp:use-mlsd",		 "no",    ResMgr::BoolValidate,0},
+   {"ftp:use-mlsd",		 "yes",   ResMgr::BoolValidate,0},
+   {"ftp:use-mode-z",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:use-pret",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-chmod",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-idle",	 "no",    ResMgr::BoolValidate,0},
@@ -338,6 +341,7 @@ static ResType lftp_vars[] = {
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:no-empty-dirs",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:require-source",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"mirror:overwrite",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
    {"sftp:auto-confirm",	 "no",	  ResMgr::BoolValidate,0},
    {"sftp:max-packets-in-flight","16",	  ResMgr::UNumberValidate,0},
@@ -385,6 +389,10 @@ static ResType lftp_vars[] = {
    {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"xfer:use-temp-file",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:temp-file-name",	 ".in.*", 0,ResMgr::NoClosure},
+   {"xfer:clobber",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:make-backup",		 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:keep-backup",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:backup-suffix",	 "~%Y%m%d%H%M%S~",0,ResMgr::NoClosure},
 
 #if USE_SSL
    {"ssl:ca-file",		 "",	  ResMgr::FileReadable,ResMgr::NoClosure},
